from typing import Literal, NotRequired, TypedDict


class NestedTimeslot(TypedDict):
    end: str
    id: int | None
    is_virtual: bool
    memo: str
    playlist_id: int | None
    repetition_of_id: int | None
    start: str


class NestedShow(TypedDict):
    default_playlist_id: int | None
    id: int
    name: str


class NestedSchedule(TypedDict):
    id: int | None
    default_playlist_id: int | None


class NestedEpisode(TypedDict):
    id: int | None
    title: str


class DayScheduleEntry(TypedDict):
    episode: NestedEpisode
    timeslot: NestedTimeslot
    show: NestedShow


class TimerangeEntry(TypedDict):
    episode: NestedEpisode
    schedule: NestedSchedule | None
    show: NestedShow
    timeslot: NestedTimeslot


class Thumbnail(TypedDict):
    width: float
    height: float
    url: str


class RadioCBASettings(TypedDict):
    api_key: NotRequired[str]
    domains: list[str]


class ProgramFallback(TypedDict):
    default_pool: str
    show_id: int | None


class MicroProgram(TypedDict):
    show_id: int | None


class RadioProgramSettings(TypedDict):
    fallback: ProgramFallback
    micro: MicroProgram


class PlayoutPools(TypedDict):
    fallback: str | None


class RadioPlayoutSettings(TypedDict):
    line_in_channels: dict[str, str]
    pools: PlayoutPools


class Logo(TypedDict):
    url: str
    height: int
    width: int


class RadioStationSettings(TypedDict):
    name: str
    logo: Logo | None
    website: str


class ImageFrame(TypedDict):
    aspect_ratio: tuple[int, int] | tuple[float, float]
    shape: Literal["rect", "round"]


class ImageRequirements(TypedDict):
    frame: ImageFrame


# done this way, because the keys have dots (".")
RadioImageRequirementsSettings = TypedDict(
    "RadioImageRequirementsSettings",
    {
        "host.image": ImageRequirements,
        "note.image": ImageRequirements,
        "show.image": ImageRequirements,
        "show.logo": ImageRequirements,
    },
)
